--- Manages the in-game log (db.epgplog)

function flsys:Log(amount, logtype, player, reason, num)
  local timestamp = time()
  table.insert(flsys.db.epgplog, 1, {timestamp, amount, logtype, player, reason, num})
  flsys.ui:LogChanged()
end


function flsys:AlterLog(log_i, raid_i, alter_type)
  --- takes form input and updates the log, ep & gp accordingly
  -- uses Undo, AddEP and AddGP to handle epgp, modifies the log itself
  -- @param alter_type - "edit" or "add"

  local amount = FLLogItemAmount:GetNumber()
  local log_type = FLLogItemType:GetText()
  local player = UIDropDownMenu_GetSelectedName(FLRosterDropDown)
  local reason = FLLogItemReason:GetText()

  if not (amount and player and reason) then
    flsys:Print("Please fill out all the fields.")
    return
  end

  if alter_type == "edit" then
    local old_item = flsys.db.epgplog[log_i]
    if amount == old_item[2] and player == old_item[4] then
      -- don't need to touch playerflsys.db, just update reason in log
      flsys.db.epgplog[log_i][5] = reason
      flsys:Print("No changes made to ep or gp.  Log modified")
      return
    else
      flsys:Undo(log_i, raid_i, false)
    end
  end

  -- add to new player
  if log_type == "EP" then
    flsys:AddEP(player, amount, reason, false, raid_i)
  elseif log_type == "GP" then
    flsys:AddGP(player, amount, reason, 0, raid_i)
  end
  if alter_type == "edit" then -- update log entry
    flsys.db.epgplog[log_i][2] = amount
    flsys.db.epgplog[log_i][4] = player
    flsys.db.epgplog[log_i][5] = reason
    flsys:Print("Change saved.")
  else -- add log entry
    -- insert before log_i so it is more recent than the raid start
    if raid_i == 1 then --chrono order for current raid
      log_i = 1
    end
    table.insert(flsys.db.epgplog, log_i, {time(), amount, log_type, player, reason})
    flsys:Print("Award added.")
  end
  FLLogItem:Hide()
  ui:UpdateLogFrame()
end

function flsys:Undo(log_i, raid_i, remove)
  --- Undo an existing EP or GP award.
  -- @remove indicates whether to delete the log entry & update everything
  --    (true when deleting, false when part of an edit)

  local _, amount, logtype, player, _, _ = unpack(flsys.db.epgplog[log_i])
  amount = flsys:Round(amount*(raidstocount + 1 - raid_i)/raidstocount, 0) --scale

  if logtype=="EP" then
    for name in string.gmatch(player, "([^, ]+)") do
      if flsys.db.players[player] then
        flsys.db.players[name].ep_total = flsys.db.players[name].ep_total - amount
        flsys.db.players[name].ep_data[raid_i] = flsys.db.players[name].ep_data[raid_i] - amount
      else
        flsys:Print("Could not undo EP award for "..player.."; they were not found in the database")
      end
    end
  elseif logtype=="GP" then
    if flsys.db.players[player] then
      flsys.db.players[player].gp_total = flsys.db.players[player].gp_total - amount
      flsys.db.players[player].gp_data[raid_i] = flsys.db.players[player].gp_data[raid_i] - amount
    else
      flsys:Print("Could not undo GP for "..player.."; they are no longer in the database.")
      return
    end
  end

  if remove then
    table.remove(flsys.db.epgplog, log_i)
    ui:UpdateLogFrame() --refresh it
    flsys:UpdatePR() --refresh standings
  end
end

function flsys:ClearLog(logtype)
  FLLog:Hide()
  if not logtype or logtype=="epgp" then
    wipe(flsys.db.epgplog)
    flsys:Print("EPGP log cleared")
  end
  -- don't need to call UpdateLog; it will get called when log is shown
end


function flsys:LogItemtoString(logitem)
  local displaystring = "  "
  local logdate, amount, logtype, players, reason, num = unpack(logitem)
  if logtype=="start" or logtype=="import" or logtype=="RESET" then
    displaystring = "|cffCCFFFF"..date("%m/%d/%y %I:%M%p", logdate).." "..reason.." "
    displaystring = displaystring.."|r"
  elseif logtype=="end" or logtype=="decay" then
    displaystring = displaystring..date("%I:%M%p", logdate).."  "..reason
  else
    displaystring = displaystring..date("%I:%M%p", logdate).."  "
    if logtype=="EP" then
      if amount>0 then amount= "+"..amount end
      displaystring = displaystring..amount.." EP "..players.." ("..reason..")"
    elseif logtype=="GP" then
      displaystring = displaystring..reason.." "..players.." ("..amount.." GP)"
    end
  end
  return displaystring
end

function flsys:ExportToString() --TODO not during combat?
  -- Provides a string that could be used to import all player epgp data
  local exportstr =""
  for name, nt in pairs(flsys.db.players) do
    exportstr = exportstr.."@"..name.."=gp:"
    for i, amt in ipairs(flsys.db.players[name].gp_data) do
      exportstr = exportstr..amt..","
    end
    exportstr = exportstr.."ep:"
    for i, amt in ipairs(flsys.db.players[name].ep_data) do
      exportstr = exportstr..amt..","
    end
  end
  return exportstr
end


StaticPopupDialogs["DELETE_LOG_ITEM"] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = "Are you sure you want to delete this award?  It will be removed from the log and the corresponding EP or GP will be removed from the player's total.",
  button1 = ACCEPT,
  button2 = CANCEL,
  OnAccept = function(self)
    flsys:Debug(1, "ALTER LOG", 'delete on accept')
    FLLogItem:Hide()
    flsys:Undo(self.log_i, self.raid_i, true)
  end,
}


StaticPopupDialogs["EXPORT_COMPLETE"] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = "Did you successfully back up your data?  If so the %s log will be cleared.",
  button1 = "Yes",
  button2 = "No",
  timeout = 0,
  whileDead = 1,
  OnAccept = function(self)
    flsys:ClearLog(self.etype)
  end,
}

