-- Handles roster, both database and in-raid
-- "roster" is made up of db.players and db.sortlist

local numraidmembers
local db

function flsys:InitializeRoster()
  db = flsys.db
end

--- TODO EVENTS notes
-- number of players changed - updatesortlist, updateraidroster if in raid,
--   checkrows, renderrows
-- PR updated - updatesortlist, renderrows
-- raid started or ended - updateraidroster, updatesortlist, checkrows

-- only relevant in a raid!
function flsys:UpdateRaidRoster()
  -- compiles a raid roster by setting inraid and adding new players as needed

  for name, _ in pairs(db.players) do
    db.players[name].inraid=nil
  end

  numraidmembers = GetNumGroupMembers()
  flsys:Debug(3, "Creating roster, "..numraidmembers.." members")
  local name
  for i = 1, numraidmembers do
    name = select(1, GetRaidRosterInfo(i))
    if db.players[name] then
      db.players[name].inraid = true
    elseif db.alts_data[name] and db.players[db.alts_data[name]] then
      flsys:NewPlayer(name, true, false, true)
    else
      flsys:NewPlayer(name, true, true)
    end
  end
  flsys:RefreshSortList()
end

-- call when db.players is altered (player added/removed, including temps/alts)
function flsys:RefreshSortList()
  --- Wipes & recreates sortlist containing all players
  wipe(db.sortlist)
  for name, t in pairs(db.players) do
    if t then
      table.insert(db.sortlist, name)
    end
  end
  flsys:SortRoster()
  flsys.ui:CheckRows()
  flsys.ui:InitializeRoster()
end


function flsys:SortRoster()
  -- sorts by: inraid > temp > PR

  flsys:Debug(2, 'SORT', 'Sorting sortlist')

  table.sort(db.sortlist, function(name1, name2)
    if db.players[name1].inraid ~= db.players[name2].inraid then
      return db.players[name1].inraid
    elseif db.players[name1].temp ~= db.players[name2].temp then
      return db.players[name2].temp
    else
      return flsys:SortByPR(name1, name2)
    end
  end)

  flsys.ui:RenderStandings()
end


function flsys:SortByPR(name1, name2)
  -- returns player with higher PR. if tied, with higher total GP.
  -- used by fld
  if db.players[name1].current_pr ~= db.players[name2].current_pr then
    return db.players[name1].current_pr > db.players[name2].current_pr
  else
    return db.players[name1].gp_total < db.players[name2].gp_total
  end
end


function flsys:GROUP_ROSTER_UPDATE()
  if not IsInRaid() then
    flsys:Debug(3, "GROUP_ROSTER_UPDATE - player no longer in raid")
    StaticPopup_Show("ENDRAID")
  elseif GetNumGroupMembers() ~= numraidmembers then
    flsys:Debug(3, "GROUP_ROSTER_UPDATE - # raid members changed")
    flsys:UpdateRaidRoster()
    fld:CreateRoster()
  end
end

-------------------------------------------------------------------------------
--- PLAYER DATABASE
-------------------------------------------------------------------------------

function flsys:NewPlayer(name, inraid, temp, alt, ep, gp)
  --- Adds a player to the database
  -- calls: RefreshSortList, ~UpdateRaid
  db.players[name] = {
    inraid = inraid,
    temp = temp or false,
    alt = alt,
    balance = 0,
    ep_data = {[1] = 0},
    gp_data = {[1] = 0}
  }
  if ep and gp then
    local ep = flsys:Round(ep*(db.raidstocount-1)/(db.raidstocount))
    local gp = flsys:Round(gp*(db.raidstocount-1)/(db.raidstocount))
    db.players[name].ep_total = ep
    db.players[name].gp_total = gp
    db.players[name].ep_data[2] = ep
    db.players[name].gp_data[2] = gp
  else
    db.players[name].ep_total = 0
    db.players[name].gp_total = 0
  end
  db.players[name].current_pr = flsys:GetPR(name)
  if db.inprogress then
    flsys:UpdateRaidRoster()
  else
    flsys:RefreshSortList()
  end
end


function flsys:AddAlt(info, input)
  local aremove, alt, main = string.match(input, "(-*)([^%d%s%p]+)=*([^%d%s%p]*)")
  if alt then alt = flsys:Cap(alt) end
  if main then main = flsys:Cap(main) end
  if aremove and db.alts_data[alt] then
    db.alts_data[alt] = nil
    flsys:Print(alt.." removed from alts list.")
  elseif alt and main then
    if not db.players[main] then
      self:Print(main.." is not in the database.  Add them before adding their alts.")
    else
      db.alts_data[alt] = main
      flsys:Print(alt.." added to alts list.")
    end
  else
    flsys:Print("Invalid entry. Make sure to format 'altname=mainname' without spaces")
  end
  alts_data = db.alts_data
end


function flsys:RemovePlayer(name)
  FLRosterFrame:Hide()
  if db.inprogress then
    self:Print("Cannot remove players during a raid.")
  else
    flsys:Debug(3, "Removing "..name.." from system.")
    db.players[name] = nil
    if db.mode==1 then self:Print(name.." removed from database.") end
    flsys:RefreshSortList()
  end
end

