flsys = LibStub('AceAddon-3.0'):NewAddon('flsys', 'AceConsole-3.0', 'AceEvent-3.0', 'AceComm-3.0')
flsys:SetDefaultModuleState(false)

local ui -- direct pointer to flsys.ui after init
local db -- direct pointer to flsys.db after init

-- cache db settings that don't change often
local raidstocount, min_gp


local DEFAULT_STANDARD_ILVL = 665

-------------------------------------------------------------------------------
--- INITIALIZE/ENABLE
-------------------------------------------------------------------------------
local defaults = {
  profile = {
    raidstocount = 15,
    min_gp = 500,
    ep_amt = 250,
    ep_recur = 30,
    players = {},
    alts_data = {},
    dates = {},
    sortlist = {},
    next_award = 0,
    lastbroadcast = 0,
    epgplog = {},
    epgp_export = 0, --timestamp
    standings_export = 0, --timestamp
    inprogress = false,
    whispopts = true,
    debuglevel = 3,
    promptrecur = true,
    mode = 1,
  },
}

function flsys:OnInitialize()
  local fulldb = LibStub('AceDB-3.0'):New('FlamingLootSysDB', defaults)
  flsys.log = LibStub('AceDB-3.0'):New('flsysdebuglog')
  flsys.db = fulldb.profile
  db = flsys.db
  ui = flsys.ui
end

function flsys:OnEnable()
  flsys:InitializeComm()
  flsys:InitializeRoster()
  flsys:RegisterChatCommand('fss', function() FLMain:Show() end)
  flsys:RegisterChatCommand('fs', 'ProcessChatCmd')
  flsys:GetConstants()
  ui:Initialize()
  if db.inprogress then -- this check needs to happen later
    flsys:InRaidCheck(true)
  end
  flsys:Print('Flaming loot masterlooter addon loaded.  Type /fs for a list of commands.')
  flsys:TrimDebug()
  flsys:CheckForGameUpdate()
end

function flsys:GetConstants()
  --load db constants into local memory
  raidstocount = db.raidstocount
  min_gp = db.min_gp
end


function flsys:CheckForGameUpdate()
  -- ensure backwards compatibility of db.game_vers and db.standard_ilvl
  -- prompt to change standard ilvl if applicable
  new_user = (db.mode == 1 and #db.sortlist == 0) --hacky way to tell if this is a new user
  if not db.standard_ilvl then
    if new_user then
      db.standard_ilvl = DEFAULT_STANDARD_ILVL
    else
      db.standard_ilvl = 496 --assume 2.3.0 default bc since then it's been set
    end
  end

  local old_vers = db.game_vers or 0
  local current = tonumber(string.gsub(select(1, GetBuildInfo()), '%.', ''))

  if db.game_vers >= old_vers and DEFAULT_STANDARD_ILVL > db.standard_ilvl then
    StaticPopup_Show('CHANGE_ILVL')
  end
end

-------------------------------------------------------------------------------
--- GP
-------------------------------------------------------------------------------
function flsys:OnLootAwarded(name, link, id, amount)
  flsys:Debug(3, 'on loot awarded called: '..name..link..amount)
  if db.players[name] then
    flsys:AddGP(name, amount, link, id)
  end
end


function flsys:AddGP(name, amount, itemname, id, raid_i)
  flsys:Debug(2, 'Adding GP: '..name..amount..itemname)
  --scale
  local r_i = raid_i or 1
  amount = flsys:Round(amount*(raidstocount + 1 - r_i)/raidstocount, 0)
  --add to total & raid
  db.players[name].gp_data[r_i] = math.max(db.players[name].gp_data[r_i] + amount, 0)
  db.players[name].gp_total = math.max(db.players[name].gp_total + amount, 0)
  --don't add log if this is retroactive; let those funcs handle it
  if not raid_i then --only for auto (not log edits)
    flsys:Log(amount, 'GP', name, itemname, id)
  end
  flsys:UpdatePR()
end


--administration
function flsys:ScaleGP(oldstd, newstd)
  local div = 2^((newstd-oldstd)/26)
  for name, nt in pairs(db.players) do
    nt.gp_total = flsys:Round(nt.gp_total/div)
    for i, g in ipairs(nt.gp_data) do
      nt.gp_data[i] = flsys:Round(g/div)
    end
  end
  flsys:Log(0, 'decay', 'all', 'GP scaled: '..oldstd..' to '..newstd, 0)
  flsys:UpdatePR()
end

function flsys:ResetGP()
  for name, nt in pairs(db.players) do
    nt.gp_total = 0
    for i, g in ipairs(nt.gp_data) do
      nt.gp_data[i] = 0
    end
  end
  flsys:Log(0, 'RESET', 'all', 'Reset all GP')
  flsys:UpdatePR()
  flsys:Print('Reset all GP.')
end

-------------------------------------------------------------------------------
--- EP
-------------------------------------------------------------------------------
function flsys:AddEP(name, amount, reason, mass, raid_i)
  flsys:Debug(2, 'Adding EP: '..name..' '..amount..'raid i '..(raid_i or 'none'))
  -- if alt, add ep to main's record
  if db.players[name].alt then
    name = db.alts_data[name]
  end
  --scale based on raid index (assume current)
  local r_i = raid_i or 1
  amount = flsys:Round(amount*(raidstocount + 1 - r_i)/raidstocount, 0) --scale
  --update raid and total
  db.players[name].ep_data[r_i] = math.max(db.players[name].ep_data[r_i] + amount, 0)
  db.players[name].ep_total = math.max(db.players[name].ep_total + amount, 0)
  --update pr, add log entry
  if not mass then
    flsys:UpdatePR()
    if not raid_i then -- only for auto awards (not log edits)
      flsys:Log(amount, 'EP', name, reason)
    end
  end
end

function flsys:MassEP(amount, reason)
  if db.inprogress then
    local namelist = ''
    for name,_ in pairs(db.players) do
      if db.players[name].inraid then
        flsys:AddEP(name, amount, reason, true)
        namelist = namelist..name..', '
      end
    end
    flsys:Log(amount, 'EP', string.sub(namelist, 1, -3), reason)
    flsys:UpdatePR()
    if db.mode == 1 then
      flsys:Print(amount..'EP awarded to raid.')
    end
  else
    flsys:Print('There is no active raid.')
  end
end
-------------------------------------------------------------------------------
--- PR & SORTING
-------------------------------------------------------------------------------
function flsys:UpdatePR()
  --- gets each players PR, calls sort, sends standings if in raid
  --BY: StartRaid, EndRaid, AddEP, AddGP, MassEP, RefreshSortList
  --CALLS: UpdateMain

  flsys:Debug(2, 'SORT', 'Updating PR for all players')
  for name, i in pairs(db.players) do
    i.current_pr = flsys:GetPR(name)
  end

  flsys:SortRoster()

  if db.inprogress then
    flsys:SendStandings('raid')
  end
end

function flsys:GetPR(name)
  --- Returns a player's PR, 0 for player's without any
  --Called by: fld
  local pr
  if not ((db.mode == 1 and db.players[name].temp) or db.players[name].alt) then
    if db.mode == 2.2 then min_gp = .5 end
    local gp = math.max(db.players[name].gp_total, min_gp)
    local ep = db.players[name].ep_total
    pr = flsys:Round(ep/gp, 2)
  else
    pr = 0
  end
  return pr
end

function flsys:DecayRaid()
  for name,_ in pairs(db.players) do
    db.players[name].ep_total = 0
    db.players[name].gp_total = 0
    local start = #db.players[name].ep_data
    if start < raidstocount then
      start = start + 1
    end
    for i = start, 2, -1 do
      db.players[name].ep_data[i] = math.floor((db.players[name].ep_data[i-1] or 0)*(raidstocount-i + 1)/(raidstocount-i + 2))
      db.players[name].gp_data[i] = math.floor((db.players[name].gp_data[i-1] or 0)*(raidstocount-i + 1)/(raidstocount-i + 2))
      db.players[name].ep_total = db.players[name].ep_total + db.players[name].ep_data[i]
      db.players[name].gp_total = db.players[name].gp_total + db.players[name].gp_data[i]
    end
    db.players[name].ep_data[1] = 0
    db.players[name].gp_data[1] = 0
  end
  flsys:Print('EP and GP decayed.')
  flsys:Log(0, 'decay', 'all', 'Decayed EP and GP')
  flsys:UpdatePR()
end
-------------------------------------------------------------------------------
--- MAJOR SETTINGS CHANGES
-------------------------------------------------------------------------------
function flsys:ModeChanged(mode)
  if mode == 1 then
    if not db.players[GetUnitName('player')] then
      flsys:NewPlayer(GetUnitName('player'))
    end
  else
    flsys:ResetData(true)
  end
end

function flsys:ResetData(removeplayers)
  wipe(db.dates)
  wipe(db.epgplog)
  if removeplayers then
    wipe(db.players)
    wipe(db.alts_data)
    flsys:Print('Removed all players and their data from the system.')
    flsys:Log(0, 'RESET', 'all', 'Reset & removed all players')
  else
    for name, ntable in pairs(db.players) do
      flsys:NewPlayer(name)
    end
    flsys:Print('Reset data for all players.')
    flsys:Log(0, 'RESET', 'all', 'Reset all EP and GP')
  end
  flsys:RefreshSortList()
end

function flsys:ImportEntered(imptext, fromplayer)
  if db.inprogress then
    flsys:Print('You cannot import or receive data while you have a raid in progress.')
    return
  end
  if imptext and imptext ~= '' then
    local printmsg, sendplayer, sendplayererror
    for psection in string.gmatch(imptext, '@([^@]+)') do
      local player = string.match(psection, '^([^=]+)')
      local eppos = string.find (psection, 'ep:')
      local balpos = string.find (psection, 'bal:') or -1
      if not player or not eppos or not string.find(psection, 'gp:') then
        printmsg = 'Import text not formatted correctly. Make sure you paste the entire text exactly as it is in the spreadsheet.  If you\'re seeing this error after receiving data from another player, please go to curse.com/addons/wow/flaming-ml and report a bug.'
        sendplayererror = 'could not import your data correctly.  This is probably an addon bug.  Please go to curse.com/addons/wow/flaming-ml and report it.'
      else
        FLImportData:Hide()
        flsys:NewPlayer(player)
        local currpart = string.sub(psection, 1, eppos)
        local pos = 1
        for amt in string.gmatch(currpart, '(%d+)') do --filling in gp data, updating total
          amt = tonumber(amt)
          db.players[player].gp_data[pos] = amt
          db.players[player].gp_total = db.players[player].gp_total +amt
          pos = pos + 1
        end
        currpart = string.sub(psection, eppos, balpos)
        pos = 1
        for amt in string.gmatch(currpart, '(%d+)') do --filling in epdata, updating total
          amt = tonumber(amt)
          db.players[player].ep_data[pos] = amt
          db.players[player].ep_total = db.players[player].ep_total +amt
          pos = pos + 1
        end
        local bal = string.match(psection, 'bal:(%-?%d+)')
        if bal then
          db.players[player].balance = tonumber(bal)
        end
        flsys:Log(0, 'import', player, 'Imported data for '..player)
        flsys:Print('Imported data for '..player..'.')
        flsys:UpdatePR()
        sendplayer = 'has imported your data!'
      end
    end
    flsys:Print(printmsg)
    if fromplayer then
      if sendplayererror then
        SendAddonMessage('flamingsys', 'datareply:'..sendplayererror, 'WHISPER', fromplayer)
      elseif sendplayer then
        SendAddonMessage('flamingsys', 'datareply:'..sendplayer, 'WHISPER', fromplayer)
      end
    end
  else
    flsys:Print('You entered a blank import.  Copy the text from your data backup spreadsheet.  If you\'re seeing this error after trying to receive data from another player, confirm that they have data stored - if so, please go to curse.com/addons/wow/flaming-ml and report a bug.')
    if fromplayer then
      SendAddonMessage('flamingsys', 'datareply: did not receive your data - either you have no data stored or there was an addon bug.  Please go to curse.com/addons/wow/flaming-ml and report it.', 'WHISPER', fromplayer)
    end
  end
end

function flsys:SendData(name)
  flsys:SendCommMessage('flamingsysdata', 'data:'..flsys:GetImportText(), 'WHISPER', name)
  flsys:Print('Sending data to '..name..'.  If they accept or decline the data you\'ll see a printout.  No further message means they don\'t have the addon turned on or they\'re using an outdated version.')
end


-------------------------------------------------------------------------------
--- POPUP CALLS TODO are these necessary, could just do _show directly?
-------------------------------------------------------------------------------
function flsys:AsktoSetupRecur()
  if db.next_award == 0 then
    StaticPopup_Show('START_RECUR')
  end
end

function flsys:CheckEndRecur()
  StaticPopup_Show('ENDRECUR')
end

function flsys:DecayPressed()
  StaticPopup_Show('DECAY')
end

function flsys:ResetDataPressed() --inprogress check done
  StaticPopup_Show('DATARESET')
end

function flsys:ResetLogPressed()
  StaticPopup_Show('LOGRESET')
end

function flsys:EndPressed()
  StaticPopup_Show('ENDRAID')
end

function flsys:RemovePlayerPressed(player)
  if player then
    local dialog = StaticPopup_Show('REMOVEPLAYER', player)
    if dialog then
      dialog.player = player
      dialog.enterClicksFirstButton = true
    end
  end
end

function flsys:ResetGPPressed()
  StaticPopup_Show('RESET_GP')
end

-------------------------------------------------------------------------------
--- POPUP DIALOGS
-------------------------------------------------------------------------------
StaticPopupDialogs['MASSEP'] = {
  text = 'Mass EP',
  button1 = 'Single',
  button2 = CANCEL,
  button3 = 'Recurring',
  hideOnEscape = 1,
  hasEditBox = 1,
  OnAccept = function(self, data)
    amt = tonumber(self.editBox:GetText())
    if amt and amt > 0 then
      flsys:MassEP(amt, db.inprogress)
    else
      self.text = 'Please enter a number amount of EP to award',
      self:Show()
    end
  end,
  OnAlt = function(self, data)
    amt = tonumber(self.editBox:GetText())
    if amt and amt > 0 then
      flsys:StartRecur(0)
    else
      self.text = 'Please enter a number amount of EP to award',
      self:Show()
    end
  end
}

StaticPopupDialogs['DATARESET'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Also remove all players from the database?',
  button1 = 'Yes',
  button2 = 'No',
  button3 = 'Cancel',
  timeout = 0,
  hideOnEscape = 1,
  whileDead = 1,
  OnAccept = function(self) flsys:ResetData(true) end,
  OnCancel = function(self) flsys:ResetData(false) end,
}

StaticPopupDialogs['LOGRESET'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Delete all log info?',
  button1 = ACCEPT,
  button2 = CANCEL,
  timeout = 0,
  hideOnEscape = 1,
  whileDead = 1,
  OnAccept = function(self) flsys:ClearLog() end,
}

StaticPopupDialogs['RESET_GP'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Reset all players\' GP to 0?  This cannot be undone.',
  button1 = ACCEPT,
  button2 = CANCEL,
  timeout = 0,
  hideOnEscape = 1,
  whileDead = 1,
  OnAccept = function(self) flsys:ResetGP() end,
}

StaticPopupDialogs['STARTRAID'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Name of raid:',
  button1 = ACCEPT,
  button2 = CANCEL,
  timeout = 0,
  whileDead = 1,
  hideOnEscape = 1,
  hasEditBox = 1,
  maxLetters = 25,
  OnAccept = function(self)
    local text = self.editBox:GetText()
    flsys:StartRaid(flsys:Cap(text))
  end
}

StaticPopupDialogs['ENDRAID'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'End the raid?',
  button1 = ACCEPT,
  button2 = CANCEL,
  hideOnEscape = 1,
  timeout = 0,
  whileDead = 1,
  OnAccept = function(self) flsys:EndRaid() end,
  OnCancel = function()
    if not fld:IsEnabled() then
      flsys:ResumeRaid()
    end
  end,
}

StaticPopupDialogs['REMOVEPLAYER'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Remove %s from the system? All of their data will be lost.',
  button1 = 'Yes',
  hideOnEscape = 1,
  button2 = 'No',
  timeout = 0,
  whileDead = 1,
  OnAccept = function(self)
    flsys:RemovePlayer(self.player)
  end,
}

StaticPopupDialogs['START_RECUR'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Number of minutes until first award should go out:',
  button1 = ACCEPT,
  button2 = CANCEL,
  timeout = 0,
  whileDead = 1,
  enterClicksFirstButton = true,
  hasEditBox = 1,
  maxLetters = 2,
  OnShow = function(self, data)
    self.editBox:SetText('0')
  end,
  OnAccept = function(self, data)
    local minutes = tonumber(self.editBox:GetText())
    flsys:StartRecur(minutes)
  end
}

StaticPopupDialogs['ENDRECUR'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'End recurring EP award?',
  button1 = 'Yes',
  button2 = 'No',
  timeout = 0,
  enterClicksFirstButton = true,
  whileDead = 1,
  hideOnEscape = 1,
  OnAccept = function(self)
    flsys:EndRecur() end,
}

StaticPopupDialogs['RESUME_RAID'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Resume your raid?',
  button1 = 'Yes',
  button2 = 'No',
  timeout = 0,
  whileDead = 1,
  OnAccept = function(self) flsys:ResumeRaid() end,
  OnCancel = function(self) flsys:EndRaid() end,
}

StaticPopupDialogs['NOTDURINGRAID'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'You cannot do that while a raid is in progress.',
  button1 = 'Ok',
  timeout = 0,
  hideOnEscape = 1,
  enterClicksFirstButton = true,
  whileDead = 1,
}

StaticPopupDialogs['CONFIRM_RAIDSTOCOUNT'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Change number of raids to count?',
  button1 = 'Yes',
  button2 = CANCEL,
  timeout = 0,
  enterClicksFirstButton = true,
  hideOnEscape = 1,
  whileDead = 1,
  OnAccept = function(self)
    flsys.db.profile.raidstocount = self.input
    flsys:GetConstants()
    end,
}

StaticPopupDialogs['DECAY'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Decay ep and gp?',
  button1 = 'Yes',
  button2 = 'No',
  timeout = 0,
  hideOnEscape = 1,
  enterClicksFirstButton = true,
  whileDead = 1,
  OnAccept = function()
    flsys:DecayRaid()
  end,
}

StaticPopupDialogs['SENDDATA'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = 'Send data to:',
  button1 = ACCEPT,
  button2 = CANCEL,
  timeout = 0,
  whileDead = 1,
  hideOnEscape = 1,
  hasEditBox = 1,
  maxLetters = 25,
  OnAccept = function(self)
    local name = self.editBox:GetText()
    flsys:SendData(name)
  end
}

StaticPopupDialogs['RECEIVE_DATA'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = '%s is sending you loot standings. Accept them?\nWARNING: Imported data overwrites stored data!',
  button1 = 'Yes',
  button2 = 'No',
  timeout = 0,
  whileDead = 1,
  OnAccept = function(self)
    flsys:ImportEntered(self.text, self.player)
  end,
  OnCancel = function(self, reason)
    SendAddonMessage('flamingsys', 'datareply:'..'declined your data.', 'WHISPER', self.player)
  end,
}

StaticPopupDialogs['CHANGE_ILVL'] = {
  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text = '|cff6600FFflaming loot|r\n\nNew patch! Update your base ilvl to match current content',
  button1 = 'Okay',
  button2 = 'No thanks',
  OnAccept = function(self)
    flsys:ProcessChatCmd('opt')
  end,
  OnCancel = function(self, reason)
    flsys:Print('GP may become disproportiately high compared to EP if you don\'t adjust it.  Type |cffffff78/fs opt|r to edit your settings later.')
  end,
}
